@@ -4,31 +4,60 @@
 #'
 #' `flextable` are designed to make tabular reporting easier for
 #' R users. Functions are available to let you format text, paragraphs and cells;
-#' table cells can be merge vertically or horizontally, row headers can easilly
+#' table cells can be merge vertically or horizontally, row headers can easily
 #' be defined, rows heights and columns widths can be manually set or automatically
 #' computed.
 #'
-#' Default formatting properties are automatically applied to every
-#' flextable you produce. You can change these default values with
-#' function [set_flextable_defaults()].
+#' If working with 'R Markdown' documents, you should read about knitr
+#' chunk options in [knit_print.flextable()] and about setting
+#' default values with [set_flextable_defaults()].
+#'
+#' @section Reuse frequently used parameters:
+#'
+#' Some default formatting properties are automatically
+#' applied to every flextable you produce.
+#'
+#' It is highly recommended to use this function because
+#' its use will minimize the code. For example, instead of
+#' calling the `fontsize()` function over and over again for
+#' each new flextable, set the font size default value by
+#' calling (before creating the flextables)
+#' `set_flextable_defaults(font.size = 11)`. This is also
+#' a simple way to have homogeneous arrays and make the
+#' documents containing them easier to read.
+#'
+#' You can change these default values with function
+#' [set_flextable_defaults()]. You can reset them
+#' with function [init_flextable_defaults()]. You
+#' can access these values by calling [get_flextable_defaults()].
+#'
+#' @section new lines and tabulations:
+#'
+#' The 'flextable' package will translate for you
+#' the new lines expressed in the form `\n` and
+#' the tabs expressed in the form `\t`.
+#'
+#' The new lines will be transformed into "soft-return",
+#' that is to say a simple carriage return and not a
+#' new paragraph.
+#'
+#' Tabs are different depending on the output format:
+#'
+#' - HTML is using entity *em space*
+#' - Word - a Word 'tab' element
+#' - PowerPoint - a PowerPoint 'tab' element
+#' - latex - tag "\\quad "
+#' @section flextable parts:
 #'
-#' @details
 #' A `flextable` is made of 3 parts: header, body and footer.
 #'
 #' Most functions have an argument named `part` that will be used
 #' to specify what part of of the table should be modified.
-#'
-#' If working with R Markdown document, you should read about knitr
-#' chunk options in [knit_print.flextable()] and about setting
-#' default values with [set_flextable_defaults()].
 #' @param data dataset
 #' @param col_keys columns names/keys to display. If some column names are not in
 #' the dataset, they will be added as blank columns by default.
 #' @param cwidth,cheight initial width and height to use for cell sizes in inches.
 #' @param defaults,theme_fun deprecated, use [set_flextable_defaults()] instead.
-#' @note Function `regulartable` is maintained for compatibility with old codes
-#' mades by users but be aware it produces the same exact object than `flextable`.
-#' This function should be deprecated then removed in the next versions.
 #' @examples
 #' ft <- flextable(head(mtcars))
 #' ft
@@ -36,11 +65,9 @@
 #' @importFrom stats setNames
 #' @seealso [style()], [autofit()], [theme_booktabs()], [knit_print.flextable()],
 #' [compose()], [footnote()], [set_caption()]
-#' @section Illustrations:
-#'
-#' \if{html}{\figure{fig_flextable_1.png}{options: width="600"}}
-flextable <- function( data, col_keys = names(data), cwidth = .75, cheight = .25,
-                       defaults = list(), theme_fun = theme_booktabs ){
+flextable <- function(data, col_keys = names(data),
+                      cwidth = .75, cheight = .25,
+                      defaults = list(), theme_fun = theme_booktabs ){
 
 
   stopifnot(is.data.frame(data), ncol(data) > 0 )
@@ -85,8 +112,9 @@ flextable <- function( data, col_keys = names(data), cwidth = .75, cheight = .25
 #' @rdname flextable
 #' @section qflextable:
 #' `qflextable` is a convenient tool to produce quickly
-#' a flextable for reporting where layoout is fixed and columns
-#' widths adjusted with [autofit()].
+#' a flextable for reporting where layout is fixed (see
+#' [set_table_properties()]) and columns
+#' widths are adjusted with [autofit()].
 qflextable <- function(data){
   ft <- flextable(data)
   ft <- set_table_properties(ft, layout = "fixed")
@@ -151,6 +179,7 @@ qflextable <- function(data){
 #' ftab <- set_caption(ftab, caption = "mtcars data", autonum = autonum)
 #' ftab
 #' @importFrom officer run_autonum
+#' @importFrom htmltools htmlEscape
 #' @seealso [flextable()]
 set_caption <- function(x, caption,
     autonum = NULL, style = "Table Caption",
@@ -174,7 +203,15 @@ set_caption <- function(x, caption,
   x
 }
 
-#' @rdname flextable
+#' @keywords internal
+#' @title flextable old functions
+#' @description The function is maintained for compatibility with old codes
+#' mades by users but be aware it produces the same exact object than [flextable()].
+#' This function should be deprecated then removed in the next versions.
+#' @param data dataset
+#' @param col_keys columns names/keys to display. If some column names are not in
+#' the dataset, they will be added as blank columns by default.
+#' @param cwidth,cheight initial width and height to use for cell sizes in inches.
 #' @export
 regulartable <- function( data, col_keys = names(data), cwidth = .75, cheight = .25 ){
   flextable(data = data, col_keys = col_keys, cwidth = cwidth, cheight = cheight)
