@@ -43,9 +43,17 @@ fit_to_width <- function(x, max_width, inc = 1L, max_iter = 20, unit = "in" ){
 
 
 #' @export
-#' @title Set flextable columns width
-#' @description control columns width
-#' @param x flextable object
+#' @title Set columns width
+#' @description Defines the widths of one or more columns in the
+#' table. This function will have no effect if you have
+#' used `set_table_properties(layout = "autofit")`.
+#'
+#' [set_table_properties()] can provide an alternative to fixed-width layouts
+#' that is supported with HTML and Word output that can be set
+#' with `set_table_properties(layout = "autofit")`.
+#'
+#'
+#' @param x a [flextable()] object
 #' @param j columns selection
 #' @param width width in inches
 #' @param unit unit for width, one of "in", "cm", "mm".
@@ -125,7 +133,7 @@ height <- function(x, i = NULL, height, part = "body", unit = "in"){
 #' @title Set flextable rule for rows heights
 #' @description control rules of each height for a part
 #' of the flextable, this is only for Word and PowerPoint outputs, it
-#' will not have any effect when output is HTML.
+#' will not have any effect when output is HTML or PDF.
 #' @param x flextable object
 #' @param i rows selection
 #' @param rule specify the meaning of the height. Possible values
@@ -271,15 +279,19 @@ dim.flextable <- function(x){
 #' @param x flextable object
 #' @param part partname of the table (one of 'all', 'body', 'header' or 'footer')
 #' @param unit unit for returned values, one of "in", "cm", "mm".
-#' @section line breaks:
-#' Soft returns (a line break in a paragraph) are not supported. Function
-#' `dim_pretty` will return wrong results if `\n` are used (they will be
-#' considered as "").
+#' @param hspans specifies how cells that are horizontally are included in the calculation.
+#' It must be one of the following values "none", "divided" or "included". If
+#' "none", widths of horizontally spanned cells is set to 0 (then do not affect the
+#' widths); if "divided", widths of horizontally spanned cells is divided by
+#' the number of spanned cells; if "included", all widths (included horizontally
+#' spanned cells) will be used in the calculation.
 #' @examples
 #' ftab <- flextable(head(mtcars))
 #' dim_pretty(ftab)
 #' @family flextable dimensions
-dim_pretty <- function( x, part = "all", unit = "in" ){
+dim_pretty <- function( x, part = "all", unit = "in", hspans = "none"){
+
+  stopifnot(length(hspans) == 1, hspans %in% c("none", "divided", "included"))
 
   part <- match.arg(part, c("all", "body", "header", "footer"), several.ok = TRUE )
   if( "all" %in% part ){
@@ -288,7 +300,7 @@ dim_pretty <- function( x, part = "all", unit = "in" ){
   dimensions <- list()
   for(j in part){
     if( nrow_part(x, j ) > 0 ){
-      dimensions[[j]] <- optimal_sizes(x[[j]])
+      dimensions[[j]] <- optimal_sizes(x[[j]], hspans = hspans)
     } else {
       dimensions[[j]] <- list(widths = rep(0, length(x$col_keys) ),
            heights = numeric(0) )
@@ -312,21 +324,31 @@ dim_pretty <- function( x, part = "all", unit = "in" ){
 #' in inches returned by function [dim_pretty()]).
 #'
 #' This function is to be used when the table widths and heights
-#' should automatically be adjusted to fit the size of the content.
+#' should be adjusted to fit the size of the content.
 #'
-#' @note
-#' This function is not related to 'Microsoft Word' *Autofit* feature.
+#' The function does not let you adjust a content that is too
+#' wide in a paginated document. It simply calculates the width
+#' of the columns so that each content has the minimum width
+#' necessary to display the content on one line.
 #'
-#' @section line breaks:
-#' Soft returns (a line break in a paragraph) are not supported. Function
-#' `autofit` will return wrong results if `\n` are used (they will be
-#' considered as "").
+#' Note that this function is not related to 'Microsoft Word'
+#' *Autofit* feature.
 #'
+#' There is an alternative to fixed-width layouts that works
+#' well with HTML and Word output that can be set
+#' with `set_table_properties(layout = "autofit")`, see
+#' [set_table_properties()].
 #' @param x flextable object
 #' @param add_w extra width to add in inches
 #' @param add_h extra height to add in inches
 #' @param unit unit for add_h and add_w, one of "in", "cm", "mm".
 #' @param part partname of the table (one of 'all', 'body', 'header' or 'footer')
+#' @param hspans specifies how cells that are horizontally are included in the calculation.
+#' It must be one of the following values "none", "divided" or "included". If
+#' "none", widths of horizontally spanned cells is set to 0 (then do not affect the
+#' widths); if "divided", widths of horizontally spanned cells is divided by
+#' the number of spanned cells; if "included", all widths (included horizontally
+#' spanned cells) will be used in the calculation.
 #' @examples
 #' ft_1 <- flextable(head(mtcars))
 #' ft_1
@@ -337,20 +359,22 @@ dim_pretty <- function( x, part = "all", unit = "in" ){
 #'
 #' \if{html}{\figure{fig_autofit_1.png}{options: width="500"}}
 #'
-#' \if{html}{\figure{fig_autofit_2.png}{options: width="400"}}
-autofit <- function(x, add_w = 0.1, add_h = 0.1, part = c("body", "header"), unit = "in"){
+#' \if{html}{\figure{fig_autofit_2.png}{options: width="379"}}
+autofit <- function(x, add_w = 0.1, add_h = 0.1, part = c("body", "header"),
+                    unit = "in", hspans = "none"){
 
   add_w <- convin(unit = unit, x = add_w)
   add_h <- convin(unit = unit, x = add_h)
 
   stopifnot(inherits(x, "flextable") )
+  stopifnot(length(hspans) == 1, hspans %in% c("none", "divided", "included"))
 
   parts <- match.arg(part, c("all", "body", "header", "footer"), several.ok = TRUE )
   if( "all" %in% parts ){
     parts <- c("header", "body", "footer")
   }
 
-  dimensions_ <- dim_pretty(x, part = parts)
+  dimensions_ <- dim_pretty(x, part = parts, hspans = hspans)
   names(dimensions_$widths) <- x$col_keys
 
   nrows <- lapply(parts, function(j){
@@ -374,9 +398,26 @@ autofit <- function(x, add_w = 0.1, add_h = 0.1, part = c("body", "header"), uni
 
 
 #' @importFrom gdtools m_str_extents
-optimal_sizes <- function( x ){
+optimal_sizes <- function( x, hspans = "none" ){
 
   sizes <- text_metric(x)
+
+  nr <- nrow(x$spans$rows)
+  rowspan <- data.frame(
+    col_id = rep(x$col_keys, each = nr),
+    ft_row_id = rep(seq_len(nr), length(x$col_keys)),
+    rowspan = as.vector(x$spans$rows),
+    stringsAsFactors = FALSE, check.names = FALSE
+  )
+  sizes <- merge(sizes, rowspan, by = c("col_id", "ft_row_id"))
+
+  if (hspans %in% "none") {
+    sizes[sizes$rowspan != 1, "width"] <- 0
+  } else if (hspans %in% "divided") {
+    sizes[sizes$rowspan > 1, "width"] <- sizes[sizes$rowspan > 1, "width"] / sizes[sizes$rowspan > 1, "rowspan"]
+    sizes[sizes$rowspan < 1, "width"] <- 0
+  }
+
   sizes$col_id <- factor(sizes$col_id, levels = x$col_keys)
   sizes <- sizes[order(sizes$col_id, sizes$ft_row_id ), ]
   widths <- as_wide_matrix_(data = sizes[, c("col_id", "width", "ft_row_id")], idvar = "ft_row_id", timevar = "col_id")
@@ -388,11 +429,6 @@ optimal_sizes <- function( x ){
   widths <- widths + par_dim$widths
   heights <- heights + par_dim$heights
 
-  widths[x$spans$rows<1] <- 0
-  widths[x$spans$columns<1] <- 0
-  heights[x$spans$rows<1] <- 0
-  heights[x$spans$columns<1] <- 0
-
   cell_dim <- dim_cells(x)
   widths <- widths + cell_dim$widths
   heights <- heights + cell_dim$heights
@@ -479,31 +515,65 @@ dim_cells <- function(x){
 }
 
 
-text_metric <- function( x ){
-  txt_data <- fortify_content(x$content, default_chunk_fmt = x$styles$text)
+text_metric <- function(x) {
+  txt_data <- fortify_content(
+    x = x$content,
+    default_chunk_fmt = x$styles$text
+  )
 
   widths <- txt_data$width
   heights <- txt_data$height
-  txt_data$width <- NULL
-  txt_data$height <- NULL
+
+  setDT(txt_data)
+
+  txt_data[, c("width", "height") := list(NULL, NULL)]
+
+  # build a fake_row_id so that new lines are considered
+  txt_data[, c("fake_row_id") := list(
+    fcase(.SD$txt %in% "<br>", 1L, default = 0L)
+  )]
+  txt_data[, c("fake_row_id") := list(
+    cumsum(.SD$fake_row_id)
+  ), by = c("ft_row_id", "col_id")]
+  txt_data[, c("fake_row_id") := list(
+    rleid(.SD$fake_row_id)
+  ), by = c("ft_row_id", "col_id")]
+
+
+  # set new lines to blank
+  txt_data$txt[txt_data$txt %in% "<br>"] <- ""
+  # set tabs to 4 'm'
+  txt_data$txt[txt_data$txt %in% "<tab>"] <- "mmmm"
 
   fontsize <- txt_data$font.size
-  fontsize[!(txt_data$vertical.align %in% "baseline")] <- fontsize[!(txt_data$vertical.align %in% "baseline")]/2
-  str_extents_ <- m_str_extents(txt_data$txt, fontname = txt_data$font.family,
-                fontsize = fontsize, bold = txt_data$bold,
-                italic = txt_data$italic) / 72
-  str_extents_[,1] <- ifelse(is.na(str_extents_[,1]) & !is.null(widths), widths, str_extents_[,1] )
-  str_extents_[,2] <- ifelse(is.na(str_extents_[,2]) & !is.null(heights), heights, str_extents_[,2] )
-  dimnames(str_extents_) <- list(NULL, c("width", "height"))
-  txt_data <- cbind( txt_data, str_extents_ )
+  not_baseline <- !(txt_data$vertical.align %in% "baseline")
+  fontsize[not_baseline] <- fontsize[not_baseline] / 2
 
-  selection_ <- c("ft_row_id", "col_id", "seq_index", "width", "height")
-  txt_data <- txt_data[, selection_]
-  setDT(txt_data)
-  txt_data <- txt_data[, c(list(width=sum(width, na.rm = TRUE), height = max(height, na.rm = TRUE) )),
-                         by= c("ft_row_id", "col_id") ]
+  extents_values <- m_str_extents(
+    txt_data$txt,
+    fontname = txt_data$font.family,
+    fontsize = fontsize,
+    bold = txt_data$bold,
+    italic = txt_data$italic
+  ) / 72
+
+  extents_values[, 1] <- ifelse(
+    is.na(extents_values[, 1]) & !is.null(widths),
+    widths, extents_values[, 1]
+  )
+  extents_values[, 2] <- ifelse(
+    is.na(extents_values[, 2]) & !is.null(heights),
+    heights, extents_values[, 2]
+  )
+  dimnames(extents_values) <- list(NULL, c("width", "height"))
+
+  txt_data <- cbind(txt_data, extents_values)
+  txt_data <- txt_data[, c(list(width = sum(.SD$width, na.rm = TRUE), height = max(.SD$height, na.rm = TRUE))),
+    by = c("ft_row_id", "fake_row_id", "col_id")
+  ]
+  txt_data <- txt_data[, c(list(width = max(.SD$width, na.rm = TRUE), height = sum(.SD$height, na.rm = TRUE))),
+    by = c("ft_row_id", "col_id")
+  ]
   setDF(txt_data)
   txt_data
 }
-
-
